#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import traceback
from notmm.utils.template import RequestContext 
from notmm.utils.wsgilib    import (
    HTTPNotModifiedResponse, 
    HTTPResponse, 
    HTTPServerError
    )

from datetime               import datetime
from time                   import ctime

from .utils                  import get_template_loader, TemplateException

TemplateLoader = get_template_loader()

__all__ = ['direct_to_template', 'render_template']

def render_template(template_name, ctx, charset='UTF-8', disable_unicode=False):
    try:
        t = TemplateLoader.get_template(template_name)
        if charset == 'UTF-8' and not disable_unicode:
            setattr(t, 'output_encoding', charset)
            chunk = t.render_unicode(data=ctx)
        else:
            chunk = t.render(data=ctx)
            
    except TemplateException as e:
        # Template error processing a unicode template
        # with Mako
        exc_type, exc_value, exc_tb = sys.exc_info()
        
        rawtb = ''.join([item for item in traceback.format_tb(exc_tb)])
        
        raise HTTPServerError(
            'error processing template: %s'
            '\n'
            '%r'
            '\n'
            '%s' % (template_name, repr(e), rawtb))
    return t, chunk

def direct_to_template(request, template_name, extra_context={}, 
    status=200, charset='UTF-8', mimetype='text/html', 
    disable_unicode=False):
    """
    Generic view for returning a Mako template inside a simple 
    ``HTTPResponse`` instance. 
    
    """
    # Make sure ctx has our stuff
    if not isinstance(extra_context, RequestContext):
        ctx = RequestContext(request, extra_context)
    else:
        ctx = extra_context

    #import pdb; pdb.set_trace()
    t, chunk = render_template(template_name, ctx)
    
    #httpheaders = HeaderDct([
    #    ('Last-Modified', datetime.fromtimestamp(t.last_modified).ctime()), 
    #    ('Date', ctime()),
    #    ('If-None-Match', str(hashlib.md5(chunk).hexdigest()))
    #    ])
    
    #print httpheaders
    #request.environ.update(httpheaders)

    return HTTPResponse(content=chunk, request=request, status=status, mimetype=mimetype)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""BaseController API 0.5.3

This package provides the ``BaseController`` class
to define custom WSGI middlewares and applications.
"""

#from notmm.controllers.wsgi    import WSGIController, process_exception
#from notmm.controllers.session import SessionController
#from notmm.controllers.routing import (
#    RegexURLMap, url, include, patterns
#    ) 

__all__ = [
    #'auth', 
    'elixir', 
    'schevo', 
    'session', 
    'wsgi', 
    'i18n'
    ]




#!/usr/bin/env python
# -*- coding: utf-8 -*-

#from notmm.controllers.auth     import AuthCookieController
from notmm.controllers.wsgi import WSGIController
from notmm.utils.django_settings import LazySettings

__all__ = ['make_app']

def make_app(handlerClass=WSGIController, enable_oauth=True):
    
    # init the django settings subsystem
    settings = LazySettings()
    
    wsgi_app = handlerClass(
        settings=settings,
        app_conf={'django.settings_autoload': True,
                  'logging.disabled': False}
    )
    if enable_oauth:
        from wsgi_oauth2 import client
        google_client = client.GoogleClient(
            settings.OAUTH2_CLIENT_ID,
            access_token=settings.OAUTH2_ACCESS_TOKEN,
            scope='email', 
            redirect_url=settings.OAUTH2_REDIRECT_URL)

        wsgi_app = google_client.wsgi_middleware(wsgi_app, 
            secret=settings.SECRET_KEY, login_path=settings.OAUTH2_LOGIN_URL)

    
    return wsgi_app



#!/usr/bin/env python
"""Base model classes for django-hotsauce"""
import logging
from notmm.dbapi.orm import RelationProxy

log = logging.getLogger(__name__)

__all__ = ('ModelManager',)

class ModelManager(object):

    model = None # Category
    db_connection = None
    def __init__(self, connection=None, model=None, **kwargs):
        self.kwargs = kwargs
        if model is not None:
            self.model = model
        if connection is not None:
            self.db_connection = connection
        self.objects = RelationProxy(getattr(self.db_connection, self.model))
        
    def __str__(self):
        return "<ModelManager: %s>" % self.model
    
    def save(self, commit=True):
        lock = self.db.write_lock()
        with lock:
            #if self.initialized:
            tx = self.extent.t.create(**self.kwargs)
            if commit:
                self.db.execute(tx)
                self.db._commit()
        lock.release()
        return self




#!/usr/bin/env python
"""
Model introspection API for django-hotsauce
"""
from importlib import import_module

__all__ = ['by_module_name']

moduleList = ('configuration', 'handlers', 'signals', 'views', 'urls', 'model')

def by_module_name(modname):
    features = {}
    mod = import_module(modname)
    features['module'] = mod
    for item in moduleList:
        if hasattr(mod, item):
            features[item] = getattr(mod, item)
    return features



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""SessionController API

Copyright (c) 2007-2011 Etienne Robillard
All rights reserved.

<LICENSE=ISC>
"""
from notmm.controllers.wsgi import WSGIController

__all__ = ['SessionController']

class SessionController(WSGIController):
    """
    A simple WSGI middleware which adds a ``session`` attribute
    to the current request instance.
    """
    def __init__(self, *args, **kwargs):
        super(SessionController, self).__init__(*args, **kwargs)
        
    #If we define a BeakerController extension the following would
    #make sense:
    #def session_active(self, wsgi_app):
    #    return bool('beaker' in wsgi_app.environ.keys() == True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2007-2018 Etienne Robillard <tkadm30@yandex.ru>
# All rights reserved.
import logging, django
from importlib import import_module
from notmm.controllers.base import BaseController
from notmm.utils.django_settings import SettingsProxy
from notmm.utils.django_compat import get_resolver

_logger = logging.getLogger(__name__)

__all__ = ('WSGIController',)

try:
    django.setup()
except:
    _logger.debug("django.setup() is disabled")


class WSGIController(BaseController):

    def __init__(self,
        settings=None,          #Django like settings module (default=$DJANGO_SETTINGS_MODULE)
        #New: dict based configuration (app_conf)
        #TODO document the changes in docs/config.txt
        app_conf={
            #'wsgi.request_class'        : HTTPRequest,     #WSGI Request middleware
            #'wsgi.response_class'       : HTTPResponse,    #WSGI Response middleware
            #'django.urlconf_class'      : None,           #Django URLConf module (default=$ROOT_URLCONF)
            'django.settings_autoload'  : True,          #Set this to False to disable Django settings autoloading (True)
            'logging_conf':None,      #Logging module to handle logging events at run-time  (experimental)
            'logging_instance':_logger,
            'logging.disabled': False,  #Set this to True to disable logging (experimental)
            #'experimental.enable_pubsub': False,#Put to this to False to disable
            #'experimental.enable_epool' : False,
            #'experimental.enable_i18n'  : False,
            #'debug.sheep.mode' :    0,
            #'debug.expert.mode':    0,
            #'debug.insane.mode':    10,
            #'debug.ponies.riding.unicorns': 1,
            #'debug.bugtracker.url': '',
            #'debug.homepage.url': 'https://wiki.python.org/moin/EtienneRobillard/DjangoHotSauce',
            #'debug.i18n.enable_unicode':True,
            #'debug.config.eval_import_func':paste_eval_import,
            #'license.registered': False,
            #'license.type': 'Apache License Version 2',
            #'license.owner.name': 'Etienne Robillard',
            #'license.owner.year': _current_date.year,
            #'license.owner.month': _current_date.month,
            #'license.version': SOFTWARE_VERSION,
            #'license.url': '',
        }):

        """
        Initializes a ``BaseController`` instance for processing
        standard Django view functions and handling basic error conditions.

        Available keyword arguments:

        - ``settings``: Django settings module (optional)
        - ``app_conf``: Dict of configuration options. (optional)
        """
        
        #self._session = local()
        #self._session.environ = {}

        # license registration flag
        # self.registered = app_conf.get('license.registered', False)

        #self.manager = ThreadLocalManager(default={'request': None})
        #self._request = self.init_request(self._environ)
        # initialize Django module by importing Django settings
        # if True. (Bool)
        autoload = app_conf.get('django.settings_autoload', True)
        if settings is None:
            self.settings = SettingsProxy(autoload=autoload).get_settings()
        else:
            self.settings = settings

        # Setup basic logging to /var/log/django.log (default)
        logging_disabled = app_conf.get('logging.disabled', False)
        if not logging_disabled:
            self.logger = _logger
        else:
            self.logger = None

        # If using legacy autoload mecanism, attempt to register user-specified
        # wsgi callbacks.
        if (autoload and hasattr(self.settings, 'CUSTOM_ERROR_HANDLERS')):
            self.registerWSGIHandlers(self.settings.CUSTOM_ERROR_HANDLERS)

        #if self.debug:
        #    log.info('logging initialized!')


        # Setup the urlconf
        # XXX This should only be necessary if using Django and is not
        # required for WSGI apps.
        #_urlconf_class = app_conf.get('django.urlconf_class', None)
        #if _urlconf_class is not None:
        #    setattr(self, 'urlconf', _urlconf_class(
        #        getattr(self.settings, 'ROOT_URLCONF')))
        #else:
        #    # work around: use django.conf.settings as
        #    # our last hope to make things work. (legacy mode)
        
        if (autoload and 'ROOT_URLCONF' in self.settings):
            setattr(self, 'urlconf', self.settings.ROOT_URLCONF)
            setattr(self, 'resolver', get_resolver(self.urlconf))
            if not hasattr(self.resolver, '_urlconf_module'):
                self.resolver._urlconf_module = import_module(self.urlconf)
        else:
            self.resolver = None
            self.urlconf = None
        
        # sanity checks 
        if self.debug:
            assert self.urlconf != None, 'urlconf instance cannot be None!'
            assert self.resolver != None, 'resolver instance cannot be None!'

        # Do something with app_conf here
        self.app_conf = app_conf




#!/usr/bin/env python
# -*- coding: utf-8 -*-

__all__ = ['HTTPException', 'HTTPClientError', 'HTTPServerError', 'HTTPNotFound', 'HTTPUnauthorized']

class HTTPException(BaseException):
    pass
class HTTPClientError(HTTPException):
    pass
class HTTPUnauthorized(HTTPClientError):
    pass
class HTTPNotFound(HTTPClientError):    
    pass
class HTTPServerError(HTTPException):
    pass




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# <LICENSE=ISC>
#
"""
Authentication and Authorization API 0.5.0

This is a thin wrapper on top of LibAuthKit to allow easy
integration with Django-based apps.
"""

from authkit.authenticate import middleware as auth_middleware
from authkit.authorize import NotAuthenticatedError
from notmm.utils.wsgilib import HTTPRequest, HTTPResponse

from session import SessionController

__all__ = ['AuthCookieController', 'LoginController']

class AuthCookieController(SessionController):
    """
    Authentication controller to delegate authorization to generic
    user-defined backends.
    
    """

    request_class = HTTPRequest
    response_class = HTTPResponse

    def __init__(self, wsgi_app, auth_conf=None, **kwargs):
            
        #super(AuthCookieController, self).__init__(**kwargs)

        #put a pointer on the previous wsgi app in the stack
        #self.wsgi_app = wsgi_app

        self.application = auth_middleware(wsgi_app(),
            app_conf=auth_conf,
            cookie_secret='secret string',
            handle_httpexception=True,
            valid=self.authenticate,
            #enforce=self.auth_conf['enforce']
            )
        
        #try to enforce cookie user setter here
        #self.auth_conf_wrapper = make_cookie_user_setter(wsgi_app, auth_conf)

    def authenticate(self, username, password):
        """
        Authenticate with the provided ``username`` and ``password``. 
        
        Developers are expected to override this method in custom
        authentication subclasses.
        """

        if username == password:
            return username
        else:
            return None

LoginController = AuthCookieController



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2008-2017 Etienne Robillard <tkadm30@yandex.com>
# All rights reserved.
"""Decorator functions to interact with Schevo databases"""

from functools import wraps
from notmm.controllers.zodb import ZODBController

__all__ = ('with_schevo_database',)


def with_schevo_database(dbname='db_default', controller_class=ZODBController):
    """
    Decorator that adds a Schevo database object reference
    in the ``request.environ`` dictionary.

    """


    def decorator(view_func, **kwargs):
        @wraps(view_func, **kwargs)
        def _wrapper(*args, **kwargs):
            req = args[0]
            wsgi_app = controller_class(req, dbname)
            req.environ[wsgi_app.environ_key] = wsgi_app.db
            return view_func(req, **kwargs)
        return wraps(view_func)(_wrapper, **kwargs)
        #return _wrapper
    return decorator

